﻿/*==============================================================================*/
/*  Copyright (C) 2020, Heartland Data inc. All Rights Reserved.                */
/*                                                                              */
/*  Title  :   Data Writing Driver                                              */
/*  EventID:   Via the driver                                                   */
/*  AppVer :   DT+ Ver1.00～                                                    */
/*  FileID :   23b                                                              */
/*  Version:   1.0                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/
/*==============================================================================*/
/*  Please define DT_ADD_EVENT_INFO when you add Event Info to Test Point       */
/*==============================================================================*/
//#define DT_ADD_EVENT_INFO

/*==============================================================================*/
/*  Please set define DT_USE_CRITICALSECTION when you use critical section.     */
/*==============================================================================*/
#define	DT_USE_CRITICALSECTION

using System;
using System.IO;
using System.Threading;

public class DtTestPointDriver
{
    [System.Runtime.InteropServices.DllImport("kernel32.dll")]
    static extern bool QueryPerformanceFrequency(out long lpFrequency);
    [System.Runtime.InteropServices.DllImport("kernel32.dll")]
    static extern bool QueryPerformanceCounter(out long lpPerformanceCount);
    //-------------------------------------------------------------------------
    //  DynamicTracar MaskCode
    //-------------------------------------------------------------------------
    static private int DT_VARIABLE_BIT = 0x02;     /* Memory Dump MASK */

    static private FileStream fs = null;
    static byte[] buf = new byte[16];
    static private bool init = false;
    static private int event_id = 0;
    static private long DtStartCount;
    static private long DtEndCount;
    static private long frequency;
    static private long time = 0;
#if DT_USE_CRITICALSECTION
    static Mutex mut = new Mutex();
#endif

    //-------------------------------------------------------------------------
    //  TestPoint InsertCode Method
    //-------------------------------------------------------------------------
    /* Normal TestPoint Method */
    static public void DtTestPoint(int nBaseAddress, int nStepNumber)
    {
#if DT_USE_CRITICALSECTION
        mut.WaitOne();
#endif
        TpBusOut(nBaseAddress, nStepNumber);
#if DT_USE_CRITICALSECTION
        mut.ReleaseMutex();
#endif
    }

    /* MemoryOutput TestPoint Method */
    static public void DtTestPointValue(int nBaseAddress, int nStepNumber, byte theValue, int nLength)
    {
#if DT_USE_CRITICALSECTION
        mut.WaitOne();
#endif
        /* Call DynamicTracerDriver */
        TpBusOut(nBaseAddress | DT_VARIABLE_BIT, nStepNumber);		/* Memory Dump */

        buf[0] = 1;     /* size */
        buf[1] = 1;     /* packetNum */
        buf[2] = 0x03;
        buf[3] = 0;
        buf[4] = (byte)((theValue) & 0xff);
        buf[5] = 0x00;
        buf[6] = 0x00;
        buf[7] = 0x00;
        buf[8] = 0x00;
        buf[9] = 0x00;
        buf[10] = 0x00;
        buf[11] = 0x00;
        buf[12] = 0x00;
        buf[13] = 0x00;
        buf[14] = 0x00;
        buf[15] = 0x00;
        try
        {
            fs.Write(buf, 0, 16);
        }
        catch (Exception)
        {
        }
#if DT_USE_CRITICALSECTION
        mut.ReleaseMutex();
#endif
    }

    static public void DtTestPointValue(int nBaseAddress, int nStepNumber, char theValue, int nLength)
    {
#if DT_USE_CRITICALSECTION
        mut.WaitOne();
#endif
        /* Call DynamicTracerDriver */
        TpBusOut(nBaseAddress | DT_VARIABLE_BIT, nStepNumber);		/* Memory Dump */

        buf[0] = 2;     /* size */
        buf[1] = 1;     /* packetNum */
        buf[2] = 0x03;
        buf[3] = 0;
        buf[4] = (byte)((theValue) & 0xff);
        buf[5] = (byte)((theValue >> 8) & 0xff);
        buf[6] = 0x00;
        buf[7] = 0x00;
        buf[8] = 0x00;
        buf[9] = 0x00;
        buf[10] = 0x00;
        buf[11] = 0x00;
        buf[12] = 0x00;
        buf[13] = 0x00;
        buf[14] = 0x00;
        buf[15] = 0x00;
        try
        {
            fs.Write(buf, 0, 16);
        }
        catch (Exception)
        {
        }
#if DT_USE_CRITICALSECTION
        mut.ReleaseMutex();
#endif
    }

    static public void DtTestPointValue(int nBaseAddress, int nStepNumber, short theValue, int nLength)
    {
#if DT_USE_CRITICALSECTION
        mut.WaitOne();
#endif
        /* Call DynamicTracerDriver */
        TpBusOut(nBaseAddress | DT_VARIABLE_BIT, nStepNumber);		/* Memory Dump */

        buf[0] = 2;     /* size */
        buf[1] = 1;     /* packetNum */
        buf[2] = 0x03;
        buf[3] = 0;
        buf[4] = (byte)((theValue) & 0xff);
        buf[5] = (byte)((theValue >> 8) & 0xff);
        buf[6] = 0x00;
        buf[7] = 0x00;
        buf[8] = 0x00;
        buf[9] = 0x00;
        buf[10] = 0x00;
        buf[11] = 0x00;
        buf[12] = 0x00;
        buf[13] = 0x00;
        buf[14] = 0x00;
        buf[15] = 0x00;
        try
        {
            fs.Write(buf, 0, 16);
        }
        catch (Exception)
        {
        }
#if DT_USE_CRITICALSECTION
        mut.ReleaseMutex();
#endif
    }

    static public void DtTestPointValue(int nBaseAddress, int nStepNumber, int theValue, int nLength)
    {
#if DT_USE_CRITICALSECTION
        mut.WaitOne();
#endif
        /* Call DynamicTracerDriver */
        TpBusOut(nBaseAddress | DT_VARIABLE_BIT, nStepNumber);		/* Memory Dump */

        buf[0] = 4;     /* size */
        buf[1] = 1;     /* packetNum */
        buf[2] = 0x03;
        buf[3] = 0;
        buf[4] = (byte)((theValue) & 0xff);
        buf[5] = (byte)((theValue >> 8) & 0xff);
        buf[6] = (byte)((theValue >> 16) & 0xff);
        buf[7] = (byte)((theValue >> 24) & 0xff);
        buf[8] = 0x00;
        buf[9] = 0x00;
        buf[10] = 0x00;
        buf[11] = 0x00;
        buf[12] = 0x00;
        buf[13] = 0x00;
        buf[14] = 0x00;
        buf[15] = 0x00;
        try
        {
            fs.Write(buf, 0, 16);
        }
        catch (Exception)
        {
        }
#if DT_USE_CRITICALSECTION
        mut.ReleaseMutex();
#endif
    }

    static public void DtTestPointValue(int nBaseAddress, int nStepNumber, long theValue, int nLength)
    {
#if DT_USE_CRITICALSECTION
        mut.WaitOne();
#endif
        /* Call DynamicTracerDriver */
        TpBusOut(nBaseAddress | DT_VARIABLE_BIT, nStepNumber);		/* Memory Dump */

        buf[0] = 8;     /* size */
        buf[1] = 1;     /* packetNum */
        buf[2] = 0x03;
        buf[3] = 0;
        buf[4] = (byte)((theValue) & 0xff);
        buf[5] = (byte)((theValue >> 8) & 0xff);
        buf[6] = (byte)((theValue >> 16) & 0xff);
        buf[7] = (byte)((theValue >> 24) & 0xff);
        buf[8] = (byte)((theValue >> 32) & 0xff);
        buf[9] = (byte)((theValue >> 40) & 0xff);
        buf[10] = (byte)((theValue >> 48) & 0xff);
        buf[11] = (byte)((theValue >> 56) & 0xff);
        buf[12] = 0x00;
        buf[13] = 0x00;
        buf[14] = 0x00;
        buf[15] = 0x00;
        try
        {
            fs.Write(buf, 0, 16);
        }
        catch (Exception)
        {
        }
#if DT_USE_CRITICALSECTION
        mut.ReleaseMutex();
#endif
    }

    static public void DtTestPointValue(int nBaseAddress, int nStepNumber, bool theValue, int nLength)
    {
#if DT_USE_CRITICALSECTION
        mut.WaitOne();
#endif
        int val = theValue ? 0x100 : 0x000;
        /* Call DynamicTracerDriver */
        TpBusOut(nBaseAddress | DT_VARIABLE_BIT, nStepNumber);		/* Memory Dump */

        buf[0] = 2;     /* size */
        buf[1] = 1;     /* packetNum */
        buf[2] = 0x03;
        buf[3] = 0;
        buf[4] = (byte)(val & 0xff);
        buf[5] = (byte)((val >> 8) & 0xff);
        buf[6] = 0x00;
        buf[7] = 0x00;
        buf[8] = 0x00;
        buf[9] = 0x00;
        buf[10] = 0x00;
        buf[11] = 0x00;
        buf[12] = 0x00;
        buf[13] = 0x00;
        buf[14] = 0x00;
        buf[15] = 0x00;
        try
        {
            fs.Write(buf, 0, 16);
        }
        catch (Exception)
        {
        }
#if DT_USE_CRITICALSECTION
        mut.ReleaseMutex();
#endif
    }

    /* Normal TestPoint Method */
    static private void TpBusOut(int nAddress, int nData)
    {
        if (init == false)
        {
            TPOpenFile();
            init = true;
        }

        QueryPerformanceCounter(out DtEndCount);
        double sec = (double)(DtEndCount - DtStartCount) / ((double)frequency); /* sec*/
        time = (long)(sec * 1000.0 * 1000.0);                             /* usec */

#if DT_ADD_EVENT_INFO
        buf[0] = (byte)(nData);
        buf[1] = (byte)(nData >> 8);
        buf[2] = (byte)(nAddress);
        buf[3] = (byte)(nAddress >> 8);
        buf[4] = (byte)(time);
        buf[5] = (byte)(time >> 8);
        buf[6] = (byte)(time >> 16);
        buf[7] = (byte)(time >> 24);
        buf[8] = (byte)(time >> 32);
        buf[9] = (byte)(time >> 40);
        buf[10] = (byte)(time >> 48);
        buf[11] = (byte)(time >> 56);
        event_id = System.Threading.Thread.CurrentThread.ManagedThreadId;
        buf[12] = (byte)(event_id);
        buf[13] = (byte)(event_id >> 8);
        buf[14] = (byte)(event_id >> 16);
        buf[15] = (byte)(event_id >> 24);
#else
        buf[0] = (byte)(nData);
        buf[1] = (byte)(nData >> 8);
        buf[2] = (byte)(nAddress);
        buf[3] = (byte)(nAddress >> 8);
        buf[4] = (byte)(nAddress >> 16);
        buf[5] = (byte)(time);
        buf[6] = (byte)(time >> 8);
        buf[7] = (byte)(time >> 16);
        buf[8] = (byte)(time >> 24);
        buf[9] = (byte)(time >> 32);
        buf[10] = (byte)(time >> 40);
        buf[11] = (byte)(time >> 48);
        buf[12] = (byte)(time >> 56);
        buf[13] = 0x00;
        buf[14] = 0x00;
        buf[15] = 0x00;
#endif
        try
        {
            fs.Write(buf, 0, 16);
        }
        catch (Exception)
        {
        }
    }

    /* open file */
    static private void TPOpenFile()
    {
        try
        {
            String path = "ConvertData.DAT";
            fs = new FileStream(path, FileMode.Create);
        }
        catch (Exception)
        {
            Console.WriteLine(false);
        }
        QueryPerformanceFrequency(out frequency);
        if (!QueryPerformanceCounter(out DtStartCount))
        {
            Console.WriteLine("QueryPerformanceCounter is out of support this system.");
            return;
        }
    }

    /* close file */
    static public void TpCloseFile()
    {
        try
        {
            fs.Close();
        }
        catch (Exception)
        {
        }
    }
}
